@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2009 Collabora Ltd.
+ * Copyright (C) 2009-2011 Collabora Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -31,9 +31,10 @@
 #include <telepathy-glib/util.h>
 #include <telepathy-glib/svc-generic.h>
 
-#include <telepathy-logger/entry-internal.h>
-#include <telepathy-logger/entry-text.h>
+#include <telepathy-logger/event-internal.h>
+#include <telepathy-logger/text-event.h>
 #include <telepathy-logger/log-manager.h>
+#include <telepathy-logger/log-manager-internal.h>
 
 #include <extensions/extensions.h>
 
@@ -109,12 +110,12 @@ favourite_contact_closure_new (TplDBusService *self,
 
 
 static gboolean
-favourite_contacts_add_entry (TplDBusService *self,
+favourite_contacts_add_event (TplDBusService *self,
     const gchar *account,
     const gchar *contact_id)
 {
   GHashTable *contacts;
-  gboolean new_entry = FALSE;
+  gboolean new_event = FALSE;
   TplDBusServicePriv *priv;
 
   g_return_val_if_fail (TPL_IS_DBUS_SERVICE (self), FALSE);
@@ -133,14 +134,14 @@ favourite_contacts_add_entry (TplDBusService *self,
           (GDestroyNotify) g_free, NULL);
       g_hash_table_insert (priv->accounts_contacts_map, g_strdup (account),
           contacts);
-      new_entry = TRUE;
+      new_event = TRUE;
     }
   else if (g_hash_table_lookup (contacts, contact_id) == NULL)
     {
-      new_entry = TRUE;
+      new_event = TRUE;
     }
 
-  if (new_entry)
+  if (new_event)
     {
       /* add dummy string for the value just for the convenience of looking up
        * whether the key already exists */
@@ -148,7 +149,7 @@ favourite_contacts_add_entry (TplDBusService *self,
           GINT_TO_POINTER (TRUE));
     }
 
-  return new_entry;
+  return new_event;
 }
 
 
@@ -187,7 +188,7 @@ favourite_contacts_parse_line (TplDBusService *self,
       success = FALSE;
     }
   else
-    favourite_contacts_add_entry (self, elements[0], elements[1]);
+    favourite_contacts_add_event (self, elements[0], elements[1]);
 
   g_strfreev (elements);
 
@@ -213,9 +214,9 @@ favourite_contacts_file_read_line_cb (GObject *object,
 
   if (error != NULL)
     {
-      DEBUG ("failed to open favourite contacts file: %s", error->message);
+      g_prefix_error (&error, "failed to open favourite contacts file: ");
+      _tpl_action_chain_terminate (action_chain, error);
       g_clear_error (&error);
-      _tpl_action_chain_terminate (action_chain);
     }
   else if (line != NULL)
     {
@@ -259,9 +260,9 @@ favourite_contacts_file_open_cb (GObject *object,
     }
   else
     {
-      DEBUG ("Failed to open the favourite contacts file: %s", error->message);
+      g_prefix_error (&error, "Failed to open the favourite contacts file: ");
+      _tpl_action_chain_terminate (action_chain, error);
       g_clear_error (&error);
-      _tpl_action_chain_terminate (action_chain);
     }
 }
 
@@ -308,11 +309,13 @@ favourite_contacts_file_parsed_cb (GObject *object,
 {
   TplDBusService *self = TPL_DBUS_SERVICE (object);
   TplDBusServicePriv *priv = self->priv;
+  GError *error = NULL;
 
-  if (!_tpl_action_chain_new_finish (result))
+  if (!_tpl_action_chain_new_finish (object, result, &error))
     {
       DEBUG ("Failed to parse the favourite contacts file and/or execute "
-          "subsequent queued method calls");
+          "subsequent queued method calls: %s", error->message);
+      g_error_free (error);
     }
 
   priv->favourite_contacts_actions = NULL;
@@ -368,193 +371,6 @@ _tpl_dbus_service_new (void)
 }
 
 
-typedef struct
-{
-  TplDBusService *self;
-  TpAccount *account;
-  char *identifier;
-  gboolean is_chatroom;
-  guint lines;
-  DBusGMethodInvocation *context;
-  GPtrArray *packed;
-  GList *dates, *ptr;
-} RecentMessagesContext;
-
-static void _lookup_next_date (RecentMessagesContext *ctx);
-
-static void
-_get_messages_return (GObject *manager,
-    GAsyncResult *res,
-    gpointer user_data)
-{
-  RecentMessagesContext *ctx = user_data;
-  GList *messages, *ptr;
-  GError *error = NULL;
-
-  if (!tpl_log_manager_get_messages_for_date_finish (
-      TPL_LOG_MANAGER (manager), res, &messages, &error))
-    {
-      DEBUG ("Failed to get messages: %s", error->message);
-
-      g_clear_error (&error);
-      messages = NULL; /* just to be sure */
-    }
-
-  /* from the most recent message, backward */
-  for (ptr = g_list_last (messages);
-       ptr != NULL && ctx->lines > 0;
-       ptr = g_list_previous (ptr))
-    {
-      TplEntry *log = ptr->data;
-      const char *message = tpl_entry_text_get_message (
-          TPL_ENTRY_TEXT (log));
-      const char *sender = tpl_entity_get_identifier (
-          tpl_entry_get_sender (log));
-      gint64 timestamp = tpl_entry_get_timestamp (log);
-
-      DEBUG ("Message: %" G_GINT64_FORMAT " <%s> %s",
-          timestamp, sender, message);
-
-      g_ptr_array_add (ctx->packed, tp_value_array_build (3,
-          G_TYPE_STRING, sender,
-          G_TYPE_STRING, message,
-          G_TYPE_INT64, timestamp,
-          G_TYPE_INVALID));
-
-      ctx->lines--;
-    }
-
-  g_list_foreach (messages, (GFunc) g_object_unref, NULL);
-  g_list_free (messages);
-
-  _lookup_next_date (ctx);
-}
-
-
-static void
-_lookup_next_date (RecentMessagesContext *ctx)
-{
-  TplDBusServicePriv *priv = ctx->self->priv;
-
-  if (ctx->ptr != NULL && ctx->lines > 0)
-    {
-      GDate *date = ctx->ptr->data;
-
-      DEBUG ("Looking up date %04u-%02u-%02u", g_date_get_year (date),
-          g_date_get_month (date), g_date_get_day (date));
-
-      tpl_log_manager_get_messages_for_date_async (priv->manager,
-          ctx->account, ctx->identifier, ctx->is_chatroom, date,
-          _get_messages_return, ctx);
-
-      ctx->ptr = g_list_previous (ctx->ptr);
-    }
-  else
-    {
-      /* return and release */
-      DEBUG ("complete, returning");
-
-      g_list_foreach (ctx->dates, (GFunc) g_date_free, NULL);
-      g_list_free (ctx->dates);
-
-      tpl_svc_logger_return_from_get_recent_messages (ctx->context,
-          ctx->packed);
-
-      g_ptr_array_free (ctx->packed, TRUE);
-      g_free (ctx->identifier);
-      g_object_unref (ctx->account);
-      g_slice_free (RecentMessagesContext, ctx);
-    }
-}
-
-
-static void
-_get_dates_return (GObject *manager,
-    GAsyncResult *res,
-    gpointer user_data)
-{
-  RecentMessagesContext *ctx = user_data;
-  GError *error = NULL;
-
-  if (!tpl_log_manager_get_dates_finish (TPL_LOG_MANAGER (manager), res,
-      &ctx->dates, &error))
-    {
-      DEBUG ("Failed to get dates: %s", error->message);
-
-      dbus_g_method_return_error (ctx->context, error);
-
-      g_clear_error (&error);
-
-      g_free (ctx->identifier);
-      g_object_unref (ctx->account);
-      g_slice_free (RecentMessagesContext, ctx);
-
-      return;
-    }
-
-  ctx->ptr = g_list_last (ctx->dates);
-  ctx->packed = g_ptr_array_new_with_free_func (
-      (GDestroyNotify) g_value_array_free);
-
-  _lookup_next_date (ctx);
-}
-
-
-static void
-tpl_dbus_service_get_recent_messages (TplSvcLogger *self,
-    const gchar *account_path,
-    const gchar *identifier,
-    gboolean is_chatroom,
-    guint lines,
-    DBusGMethodInvocation *context)
-{
-  TplDBusServicePriv *priv = TPL_DBUS_SERVICE (self)->priv;
-  TpDBusDaemon *tp_dbus;
-  TpAccount *account;
-  RecentMessagesContext *ctx;
-  GError *error = NULL;
-
-  g_return_if_fail (TPL_IS_DBUS_SERVICE (self));
-  g_return_if_fail (context != NULL);
-
-  tp_dbus = tp_dbus_daemon_dup (&error);
-  if (tp_dbus == NULL)
-    {
-      DEBUG ("Unable to acquire the bus daemon: %s", error->message);
-      dbus_g_method_return_error (context, error);
-      goto out;
-    }
-
-  account = tp_account_new (tp_dbus, account_path, &error);
-  if (account == NULL)
-    {
-      DEBUG ("Unable to acquire the account for %s: %s", account_path,
-          error->message);
-      dbus_g_method_return_error (context, error);
-      goto out;
-    }
-
-  ctx = g_slice_new (RecentMessagesContext);
-  ctx->self = TPL_DBUS_SERVICE (self);
-  ctx->account = account;
-  ctx->identifier = g_strdup (identifier);
-  ctx->is_chatroom = is_chatroom;
-  ctx->lines = lines;
-  ctx->context = context;
-
-  tpl_log_manager_get_dates_async (priv->manager,
-      account, identifier, is_chatroom,
-      _get_dates_return, ctx);
-
-out:
-
-  if (tp_dbus != NULL)
-    g_object_unref (tp_dbus);
-
-  g_clear_error (&error);
-}
-
-
 static void
 append_favourite_contacts_account_and_contacts (const gchar *account,
     GHashTable *contacts,
@@ -775,7 +591,7 @@ pendingproc_add_favourite_contact (TplActionChain *action_chain,
       goto pendingproc_add_favourite_contact_ERROR;
     }
 
-  should_add = favourite_contacts_add_entry (closure->service, closure->account,
+  should_add = favourite_contacts_add_event (closure->service, closure->account,
       closure->contact_id);
 
   closure->cb = add_favourite_contact_file_save_cb;
@@ -788,9 +604,10 @@ pendingproc_add_favourite_contact (TplActionChain *action_chain,
   return;
 
 pendingproc_add_favourite_contact_ERROR:
-  g_clear_error (&error);
   if (action_chain != NULL)
-    _tpl_action_chain_terminate (action_chain);
+    _tpl_action_chain_terminate (action_chain, error);
+
+  g_clear_error (&error);
 }
 
 
@@ -888,9 +705,10 @@ pendingproc_remove_favourite_contact (TplActionChain *action_chain,
   return;
 
 pendingproc_remove_favourite_contact_ERROR:
-  g_clear_error (&error);
   if (action_chain != NULL)
-    _tpl_action_chain_terminate (action_chain);
+    _tpl_action_chain_terminate (action_chain, error);
+
+  g_clear_error (&error);
 }
 
 static void
@@ -921,6 +739,118 @@ tpl_dbus_service_remove_favourite_contact (TplSvcLogger *logger,
     pendingproc_remove_favourite_contact (NULL, closure);
 }
 
+
+static void
+tpl_dbus_service_clear (TplSvcLogger *logger,
+    DBusGMethodInvocation *context)
+{
+  TplDBusService *self = TPL_DBUS_SERVICE (logger);
+
+  g_return_if_fail (TPL_IS_DBUS_SERVICE (self));
+  g_return_if_fail (context != NULL);
+
+  /* We want to clear synchronously to avoid concurent write */
+  _tpl_log_manager_clear (self->priv->manager);
+
+  tpl_svc_logger_return_from_clear (context);
+}
+
+
+static void
+tpl_dbus_service_clear_account (TplSvcLogger *logger,
+    const gchar *account_path,
+    DBusGMethodInvocation *context)
+{
+  TplDBusService *self = TPL_DBUS_SERVICE (logger);
+  TpDBusDaemon *bus;
+  TpAccount *account;
+  GError *error = NULL;
+
+  g_return_if_fail (TPL_IS_DBUS_SERVICE (self));
+  g_return_if_fail (context != NULL);
+
+  bus = tp_dbus_daemon_dup (&error);
+  if (bus == NULL)
+    {
+      DEBUG ("Unable to acquire the bus daemon: %s", error->message);
+      dbus_g_method_return_error (context, error);
+      goto out;
+    }
+
+  account = tp_account_new (bus, account_path, &error);
+  if (account == NULL)
+    {
+      DEBUG ("Unable to acquire the account for %s: %s", account_path,
+          error->message);
+      dbus_g_method_return_error (context, error);
+      goto out;
+    }
+
+  /* We want to clear synchronously to avoid concurent write */
+  _tpl_log_manager_clear_account (self->priv->manager, account);
+  g_object_unref (account);
+
+  tpl_svc_logger_return_from_clear_account (context);
+
+out:
+  if (bus != NULL)
+    g_object_unref (bus);
+
+  g_clear_error (&error);
+}
+
+
+static void
+tpl_dbus_service_clear_entity (TplSvcLogger *logger,
+    const gchar *account_path,
+    const gchar *identifier,
+    gint type,
+    DBusGMethodInvocation *context)
+{
+  TplDBusService *self = TPL_DBUS_SERVICE (logger);
+  TpDBusDaemon *bus;
+  TpAccount *account;
+  TplEntity *entity;
+  GError *error = NULL;
+
+  g_return_if_fail (TPL_IS_DBUS_SERVICE (self));
+  g_return_if_fail (context != NULL);
+  g_return_if_fail (!TPL_STR_EMPTY (identifier));
+
+  bus = tp_dbus_daemon_dup (&error);
+  if (bus == NULL)
+    {
+      DEBUG ("Unable to acquire the bus daemon: %s", error->message);
+      dbus_g_method_return_error (context, error);
+      goto out;
+    }
+
+  account = tp_account_new (bus, account_path, &error);
+  if (account == NULL)
+    {
+      DEBUG ("Unable to acquire the account for %s: %s", account_path,
+          error->message);
+      dbus_g_method_return_error (context, error);
+      goto out;
+    }
+
+  entity = tpl_entity_new (identifier, type, NULL, NULL);
+
+  /* We want to clear synchronously to avoid concurent write */
+  _tpl_log_manager_clear_entity (self->priv->manager, account, entity);
+
+  g_object_unref (account);
+  g_object_unref (entity);
+
+  tpl_svc_logger_return_from_clear_account (context);
+
+out:
+  if (bus != NULL)
+    g_object_unref (bus);
+
+  g_clear_error (&error);
+}
+
 static void
 tpl_logger_iface_init (gpointer iface,
     gpointer iface_data)
@@ -928,9 +858,11 @@ tpl_logger_iface_init (gpointer iface,
   TplSvcLoggerClass *klass = (TplSvcLoggerClass *) iface;
 
 #define IMPLEMENT(x) tpl_svc_logger_implement_##x (klass, tpl_dbus_service_##x)
-  IMPLEMENT (get_recent_messages);
   IMPLEMENT (get_favourite_contacts);
   IMPLEMENT (add_favourite_contact);
   IMPLEMENT (remove_favourite_contact);
+  IMPLEMENT (clear);
+  IMPLEMENT (clear_account);
+  IMPLEMENT (clear_entity);
 #undef IMPLEMENT
 }
