@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2003-2007 Imendio AB
- * Copyright (C) 2007-2010 Collabora Ltd.
+ * Copyright (C) 2007-2011 Collabora Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,6 +25,7 @@
 #include "config.h"
 #include "log-store-xml-internal.h"
 
+#include <errno.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -39,16 +40,15 @@
 #include <telepathy-glib/defs.h>
 #include <telepathy-glib/util.h>
 
-#include <telepathy-logger/entry-internal.h>
-#include <telepathy-logger/entry-text.h>
-#include <telepathy-logger/entry-text-internal.h>
+#include <telepathy-logger/event-internal.h>
+#include <telepathy-logger/text-event.h>
+#include <telepathy-logger/text-event-internal.h>
 #include <telepathy-logger/log-manager.h>
 #include <telepathy-logger/log-store-internal.h>
 #include <telepathy-logger/log-manager-internal.h>
 
 #define DEBUG_FLAG TPL_DEBUG_LOG_STORE
 #include <telepathy-logger/entity-internal.h>
-#include <telepathy-logger/datetime-internal.h>
 #include <telepathy-logger/debug-internal.h>
 #include <telepathy-logger/util-internal.h>
 
@@ -74,6 +74,7 @@ struct _TplLogStoreXmlPriv
   gboolean readable;
   gboolean writable;
   gboolean empathy_legacy;
+  gboolean test_mode;
   TpAccountManager *account_manager;
 };
 
@@ -83,7 +84,8 @@ enum {
     PROP_READABLE,
     PROP_WRITABLE,
     PROP_BASEDIR,
-    PROP_EMPATHY_LEGACY
+    PROP_EMPATHY_LEGACY,
+    PROP_TESTMODE
 };
 
 static void log_store_iface_init (gpointer g_iface, gpointer iface_data);
@@ -121,6 +123,8 @@ log_store_xml_dispose (GObject *object)
       g_object_unref (priv->account_manager);
       priv->account_manager = NULL;
     }
+
+  G_OBJECT_CLASS (_tpl_log_store_xml_parent_class)->dispose (object);
 }
 
 
@@ -168,6 +172,9 @@ tpl_log_store_xml_get_property (GObject *object,
       case PROP_EMPATHY_LEGACY:
         g_value_set_boolean (value, priv->empathy_legacy);
         break;
+      case PROP_TESTMODE:
+        g_value_set_boolean (value, priv->test_mode);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
         break;
@@ -200,6 +207,9 @@ tpl_log_store_xml_set_property (GObject *object,
       case PROP_BASEDIR:
         log_store_xml_set_basedir (self, g_value_get_string (value));
         break;
+      case PROP_TESTMODE:
+        self->priv->test_mode = g_value_get_boolean (value);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
         break;
@@ -248,6 +258,12 @@ _tpl_log_store_xml_class_init (TplLogStoreXmlClass *klass)
   g_object_class_install_property (object_class, PROP_EMPATHY_LEGACY,
       param_spec);
 
+  param_spec = g_param_spec_boolean ("testmode",
+      "TestMode",
+      "Wheter the logstore is in testmode, for testsuite use only",
+      FALSE, G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_TESTMODE, param_spec);
+
   g_type_class_add_private (object_class, sizeof (TplLogStoreXmlPriv));
 }
 
@@ -275,34 +291,46 @@ log_store_account_to_dirname (TpAccount *account)
   return g_strdelimit (g_strdup (name), "/", '_');
 }
 
-/* chat_id can be NULL, but if present have to be a non zero-lenght string.
+
+/* id can be NULL, but if present have to be a non zero-lenght string.
  * If NULL, the returned dir will be composed until the account part.
- * If non-NULL, the returned dir will be composed until the chat_id part */
+ * If non-NULL, the returned dir will be composed until the id part */
 static gchar *
 log_store_xml_get_dir (TplLogStoreXml *self,
     TpAccount *account,
-    const gchar *chat_id,
-    gboolean chatroom)
+    TplEntity *target)
 {
   gchar *basedir;
-  gchar *escaped;
+  gchar *escaped_account;
+  gchar *escaped_id = NULL;
 
   g_return_val_if_fail (TPL_IS_LOG_STORE_XML (self), NULL);
   g_return_val_if_fail (TP_IS_ACCOUNT (account), NULL);
-  /* chat_id may be NULL, but not empthy string if not-NULL */
-  g_return_val_if_fail ((chat_id == NULL) || (*chat_id != '\0'), NULL);
 
-  escaped = log_store_account_to_dirname (account);
+  escaped_account = log_store_account_to_dirname (account);
 
-  if (chatroom)
+  if (target)
+    {
+      /* FIXME This may be source of bug (does that case still exist ?)
+       * avoid that 1-1 conversation generated from a chatroom, having id similar
+       * to room@conference.domain/My_Alias (in XMPP) are treated as a directory
+       * path, creating My_Alias as a subdirectory of room@conference.domain */
+      escaped_id = g_strdelimit (
+          g_strdup (tpl_entity_get_identifier (target)),
+          "/", '_');
+    }
+
+  if (target != NULL
+      && tpl_entity_get_entity_type (target) == TPL_ENTITY_ROOM)
     basedir = g_build_path (G_DIR_SEPARATOR_S,
-        log_store_xml_get_basedir (self), escaped, LOG_DIR_CHATROOMS,
-        chat_id, NULL);
+        log_store_xml_get_basedir (self), escaped_account, LOG_DIR_CHATROOMS,
+        escaped_id, NULL);
   else
     basedir = g_build_path (G_DIR_SEPARATOR_S,
-        log_store_xml_get_basedir (self), escaped, chat_id, NULL);
+        log_store_xml_get_basedir (self), escaped_account, escaped_id, NULL);
 
-  g_free (escaped);
+  g_free (escaped_account);
+  g_free (escaped_id);
 
   return basedir;
 }
@@ -311,14 +339,15 @@ log_store_xml_get_dir (TplLogStoreXml *self,
 static gchar *
 log_store_xml_get_timestamp_filename (void)
 {
-  time_t t;
   gchar *time_str;
   gchar *filename;
+  GDateTime *now;
 
-  t = _tpl_time_get_current ();
-  time_str = _tpl_time_to_string_local (t, LOG_TIME_FORMAT);
+  now = g_date_time_new_now_local ();
+  time_str = g_date_time_format (now, LOG_TIME_FORMAT);
   filename = g_strconcat (time_str, LOG_FILENAME_SUFFIX, NULL);
 
+  g_date_time_unref (now);
   g_free (time_str);
 
   return filename;
@@ -326,74 +355,69 @@ log_store_xml_get_timestamp_filename (void)
 
 
 static gchar *
-log_store_xml_get_timestamp_from_message (TplEntry *message)
+log_store_xml_get_timestamp_from_event (TplEvent *event)
 {
-  time_t t;
+  GDateTime *ts;
+  gchar *ts_str;
 
-  t = tpl_entry_get_timestamp (message);
+  ts = g_date_time_new_from_unix_utc (tpl_event_get_timestamp (event));
+  ts_str = g_date_time_format (ts, LOG_TIME_FORMAT_FULL);
 
-  /* We keep the timestamps in the messages as UTC */
-  return _tpl_time_to_string_utc (t, LOG_TIME_FORMAT_FULL);
+  g_date_time_unref (ts);
+
+  return ts_str;
 }
 
 
 static gchar *
 log_store_xml_get_filename (TplLogStoreXml *self,
     TpAccount *account,
-    const gchar *chat_id,
-    gboolean chatroom)
+    TplEntity *target)
 {
-  gchar *chatid_dir;
+  gchar *id_dir;
   gchar *timestamp;
   gchar *filename;
-  gchar *esc_chat_id;
 
-  /* avoid that 1-1 conversation generated from a chatroom, having id similar
-   * to room@conference.domain/My_Alias (in XMPP) are threated as a directory
-   * path, creating My_Alias as a subdirectory of room@conference.domain */
-  esc_chat_id = g_strdelimit (g_strdup (chat_id), "/", '_');
-  chatid_dir = log_store_xml_get_dir (self, account, esc_chat_id,
-      chatroom);
+  id_dir = log_store_xml_get_dir (self, account, target);
   timestamp = log_store_xml_get_timestamp_filename ();
-  filename = g_build_filename (chatid_dir, timestamp, NULL);
+  filename = g_build_filename (id_dir, timestamp, NULL);
 
-  g_free (esc_chat_id);
-  g_free (chatid_dir);
+  g_free (id_dir);
   g_free (timestamp);
 
   return filename;
 }
 
 
-/* this is a method used at the end of the add_message process, used by any
- * Entry<Type> instance. it should the only method allowed to write to the
+/* this is a method used at the end of the add_event process, used by any
+ * Event<Type> instance. it should the only method allowed to write to the
  * store */
 static gboolean
 _log_store_xml_write_to_store (TplLogStoreXml *self,
     TpAccount *account,
-    const gchar *chat_id,
-    gboolean chatroom,
-    const gchar *entry,
+    TplEntity *target,
+    const gchar *event,
     GError **error)
 {
   FILE *file;
   gchar *filename;
   gchar *basedir;
+  gboolean ret = TRUE;
 
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
   g_return_val_if_fail (TPL_IS_LOG_STORE_XML (self), FALSE);
   g_return_val_if_fail (TP_IS_ACCOUNT (account), FALSE);
-  g_return_val_if_fail (!TPL_STR_EMPTY (chat_id), FALSE);
-  g_return_val_if_fail (!TPL_STR_EMPTY (entry), FALSE);
+  g_return_val_if_fail (TPL_IS_ENTITY (target), FALSE);
 
-  filename = log_store_xml_get_filename (self, account, chat_id,
-      chatroom);
+  filename = log_store_xml_get_filename (self, account, target);
   basedir = g_path_get_dirname (filename);
+
   if (!g_file_test (basedir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))
     {
       DEBUG ("Creating directory: '%s'", basedir);
       g_mkdir_with_parents (basedir, LOG_DIR_CREATE_MODE);
     }
+
   g_free (basedir);
 
   if (!g_file_test (filename, G_FILE_TEST_EXISTS))
@@ -410,18 +434,29 @@ _log_store_xml_write_to_store (TplLogStoreXml *self,
       if (file != NULL)
         fseek (file, -strlen (LOG_FOOTER), SEEK_END);
     }
-  g_fprintf (file, "%s", entry);
-  /*DEBUG ("%s: written: %s", filename, entry);*/
+
+  if (file == NULL)
+    {
+      g_set_error (error, TPL_LOG_STORE_ERROR,
+          TPL_LOG_STORE_ERROR_FAILED,
+          "Couldn't open log file: %s", filename);
+      ret = FALSE;
+      goto out;
+    }
+
+  g_fprintf (file, "%s", event);
+  DEBUG ("%s: written: %s", filename, event);
 
   fclose (file);
+ out:
   g_free (filename);
-  return TRUE;
+  return ret;
 }
 
 
 static gboolean
-add_message_text_chat (TplLogStoreXml *self,
-    TplEntryText *message,
+add_text_event (TplLogStoreXml *self,
+    TplTextEvent *message,
     GError **error)
 {
   gboolean ret = FALSE;
@@ -430,16 +465,16 @@ add_message_text_chat (TplLogStoreXml *self,
   TplEntity *sender;
   const gchar *body_str;
   gchar *avatar_token = NULL;
-  gchar *body;
-  gchar *timestamp;
+  gchar *body = NULL;
+  gchar *timestamp = NULL;
   gchar *contact_name = NULL;
-  gchar *contact_id;
-  gchar *entry;
+  gchar *contact_id = NULL;
+  gchar *event = NULL;
   TpChannelTextMessageType msg_type;
 
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
   g_return_val_if_fail (TPL_IS_LOG_STORE_XML (self), FALSE);
-  g_return_val_if_fail (TPL_IS_ENTRY_TEXT (message), FALSE);
+  g_return_val_if_fail (TPL_IS_TEXT_EVENT (message), FALSE);
 
   bus_daemon = tp_dbus_daemon_dup (error);
   if (bus_daemon == NULL)
@@ -448,18 +483,23 @@ add_message_text_chat (TplLogStoreXml *self,
       goto out;
     }
 
-  account =  tpl_entry_get_account (TPL_ENTRY (message));
+  account =  tpl_event_get_account (TPL_EVENT (message));
 
-  body_str = tpl_entry_text_get_message (message);
+  body_str = tpl_text_event_get_message (message);
   if (TPL_STR_EMPTY (body_str))
-    goto out;
+    {
+      g_set_error (error, TPL_LOG_STORE_ERROR,
+          TPL_LOG_STORE_ERROR_FAILED,
+          "The message body is empty or NULL");
+      goto out;
+    }
 
   body = g_markup_escape_text (body_str, -1);
-  msg_type = _tpl_entry_text_get_message_type (message);
-  timestamp = log_store_xml_get_timestamp_from_message (
-      TPL_ENTRY (message));
+  msg_type = tpl_text_event_get_message_type (message);
+  timestamp = log_store_xml_get_timestamp_from_event (
+      TPL_EVENT (message));
 
-  sender = tpl_entry_get_sender (TPL_ENTRY (message));
+  sender = tpl_event_get_sender (TPL_EVENT (message));
   contact_id = g_markup_escape_text (tpl_entity_get_identifier (sender), -1);
   if (tpl_entity_get_alias (sender) != NULL)
     contact_name = g_markup_escape_text (tpl_entity_get_alias (sender), -1);
@@ -467,32 +507,28 @@ add_message_text_chat (TplLogStoreXml *self,
     avatar_token = g_markup_escape_text (tpl_entity_get_avatar_token
         (sender), -1);
 
-  entry = g_strdup_printf ("<message time='%s' cm_id='%s' id='%s' name='%s' "
+  event = g_strdup_printf ("<message time='%s' id='%s' name='%s' "
       "token='%s' isuser='%s' type='%s'>"
       "%s</message>\n" LOG_FOOTER, timestamp,
-      _tpl_entry_get_log_id (TPL_ENTRY (message)),
       contact_id, contact_name,
       avatar_token ? avatar_token : "",
-      tpl_entity_get_entity_type (sender) ==
-      TPL_ENTITY_SELF ? "true" : "false",
-      _tpl_entry_text_message_type_to_str (msg_type),
+      tpl_entity_get_entity_type (sender)
+          == TPL_ENTITY_SELF ? "true" : "false",
+      _tpl_text_event_message_type_to_str (msg_type),
       body);
 
-  DEBUG ("writing %s from %s (ts %s)",
-      _tpl_entry_get_log_id (TPL_ENTRY (message)),
+  DEBUG ("writing event from %s (ts %s)",
       contact_id, timestamp);
 
   ret = _log_store_xml_write_to_store (self, account,
-      _tpl_entry_get_chat_id (TPL_ENTRY (message)),
-      _tpl_entry_text_is_chatroom (message),
-      entry, error);
+      _tpl_event_get_target (TPL_EVENT (message)), event, error);
 
 out:
   g_free (contact_id);
   g_free (contact_name);
   g_free (timestamp);
   g_free (body);
-  g_free (entry);
+  g_free (event);
   g_free (avatar_token);
 
   if (bus_daemon != NULL)
@@ -502,75 +538,33 @@ out:
 }
 
 
+/* First of two phases selection: understand the type Event */
 static gboolean
-add_message_text (TplLogStoreXml *self,
-    TplEntryText *message,
-    GError **error)
-{
-  TplEntryTextSignalType signal_type;
-
-  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-  g_return_val_if_fail (TPL_IS_LOG_STORE_XML (self), FALSE);
-  g_return_val_if_fail (TPL_IS_ENTRY_TEXT (message), FALSE);
-
-  signal_type = _tpl_entry_get_signal_type (TPL_ENTRY (message));
-
-  switch (signal_type)
-    {
-      case TPL_ENTRY_TEXT_SIGNAL_SENT:
-      case TPL_ENTRY_TEXT_SIGNAL_RECEIVED:
-        return add_message_text_chat (self, message, error);
-        break;
-      case TPL_ENTRY_TEXT_SIGNAL_CHAT_STATUS_CHANGED:
-        g_warning ("STATUS_CHANGED log entry not currently handled");
-        return FALSE;
-        break;
-      case TPL_ENTRY_TEXT_SIGNAL_SEND_ERROR:
-        g_warning ("SEND_ERROR log entry not currently handled");
-        return FALSE;
-      case TPL_ENTRY_TEXT_SIGNAL_LOST_MESSAGE:
-        g_warning ("LOST_MESSAGE log entry not currently handled");
-        return FALSE;
-      default:
-        g_warning ("Entry's signal type unknown");
-        return FALSE;
-    }
-}
-
-
-/* First of two phases selection: understand the type Entry */
-static gboolean
-log_store_xml_add_message (TplLogStore *store,
-    TplEntry *message,
+log_store_xml_add_event (TplLogStore *store,
+    TplEvent *event,
     GError **error)
 {
   TplLogStoreXml *self = TPL_LOG_STORE_XML (store);
 
-  g_return_val_if_fail (TPL_IS_ENTRY (message), FALSE);
+  g_return_val_if_fail (TPL_IS_EVENT (event), FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  switch (_tpl_entry_get_signal_type (TPL_ENTRY (message)))
-    {
-      case TPL_ENTRY_CHANNEL_TEXT_SIGNAL_SENT:
-      case TPL_ENTRY_CHANNEL_TEXT_SIGNAL_RECEIVED:
-      case TPL_ENTRY_CHANNEL_TEXT_SIGNAL_SEND_ERROR:
-      case TPL_ENTRY_CHANELL_TEXT_SIGNAL_LOST_MESSAGE:
-        return add_message_text (self, TPL_ENTRY_TEXT (message), error);
-      default:
-        DEBUG ("TplEntrySignalType not handled by this LogStore (%s). "
-            "Ignoring Entry", log_store_xml_get_name (store));
-        /* do not consider it an error, this LogStore simply do not want/need
-         * this Entry */
-        return TRUE;
-    }
+  if (TPL_IS_TEXT_EVENT (event))
+    return add_text_event (self, TPL_TEXT_EVENT (event), error);
+
+  DEBUG ("TplEntry not handled by this LogStore (%s). "
+      "Ignoring Event", log_store_xml_get_name (store));
+  /* do not consider it an error, this LogStore simply do not want/need
+   * this Event */
+  return TRUE;
 }
 
 
 static gboolean
 log_store_xml_exists (TplLogStore *store,
     TpAccount *account,
-    const gchar *chat_id,
-    gboolean chatroom)
+    TplEntity *target,
+    gint type_mask)
 {
   TplLogStoreXml *self = (TplLogStoreXml *) store;
   gchar *dir;
@@ -578,9 +572,12 @@ log_store_xml_exists (TplLogStore *store,
 
   g_return_val_if_fail (TPL_IS_LOG_STORE_XML (self), FALSE);
   g_return_val_if_fail (TP_IS_ACCOUNT (account), FALSE);
-  g_return_val_if_fail (!TPL_STR_EMPTY (chat_id), FALSE);
+  g_return_val_if_fail (TPL_IS_ENTITY (target), FALSE);
 
-  dir = log_store_xml_get_dir (self, account, chat_id, chatroom);
+  if (!(type_mask & TPL_EVENT_MASK_TEXT))
+    return FALSE;
+
+  dir = log_store_xml_get_dir (self, account, target);
   exists = g_file_test (dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR);
   g_free (dir);
 
@@ -612,8 +609,8 @@ create_date_from_string (const gchar *str)
 static GList *
 log_store_xml_get_dates (TplLogStore *store,
     TpAccount *account,
-    const gchar *chat_id,
-    gboolean chatroom)
+    TplEntity *target,
+    gint type_mask)
 {
   TplLogStoreXml *self = (TplLogStoreXml *) store;
   GList *dates = NULL;
@@ -624,9 +621,12 @@ log_store_xml_get_dates (TplLogStore *store,
 
   g_return_val_if_fail (TPL_IS_LOG_STORE_XML (self), NULL);
   g_return_val_if_fail (TP_IS_ACCOUNT (account), NULL);
-  g_return_val_if_fail (!TPL_STR_EMPTY (chat_id), NULL);
+  g_return_val_if_fail (TPL_IS_ENTITY (target), NULL);
 
-  directory = log_store_xml_get_dir (self, account, chat_id, chatroom);
+  if (!(type_mask & TPL_EVENT_MASK_TEXT))
+    return NULL;
+
+  directory = log_store_xml_get_dir (self, account, target);
   dir = g_dir_open (directory, 0, NULL);
   if (!dir)
     {
@@ -671,8 +671,7 @@ log_store_xml_get_dates (TplLogStore *store,
 static gchar *
 log_store_xml_get_filename_for_date (TplLogStoreXml *self,
     TpAccount *account,
-    const gchar *chat_id,
-    gboolean chatroom,
+    TplEntity *target,
     const GDate *date)
 {
   gchar *basedir;
@@ -682,12 +681,12 @@ log_store_xml_get_filename_for_date (TplLogStoreXml *self,
 
   g_return_val_if_fail (TPL_IS_LOG_STORE_XML (self), NULL);
   g_return_val_if_fail (TP_IS_ACCOUNT (account), NULL);
-  g_return_val_if_fail (!TPL_STR_EMPTY (chat_id), NULL);
+  g_return_val_if_fail (TPL_IS_ENTITY (target), NULL);
   g_return_val_if_fail (date != NULL, NULL);
 
   g_date_strftime (str, 9, "%Y%m%d", date);
 
-  basedir = log_store_xml_get_dir (self, account, chat_id, chatroom);
+  basedir = log_store_xml_get_dir (self, account, target);
   timestamp = g_strconcat (str, LOG_FILENAME_SUFFIX, NULL);
   filename = g_build_filename (basedir, timestamp, NULL);
 
@@ -713,6 +712,7 @@ log_store_xml_search_hit_new (TplLogStoreXml *self,
   GDate *date;
   const gchar *chat_id;
   gboolean is_chatroom;
+  TplEntity *target;
 
   g_return_val_if_fail (TPL_IS_LOG_STORE_XML (self), NULL);
   g_return_val_if_fail (!TPL_STR_EMPTY (filename), NULL);
@@ -751,26 +751,37 @@ log_store_xml_search_hit_new (TplLogStoreXml *self,
     }
   g_list_free (accounts);
 
-  hit = _tpl_log_manager_search_hit_new (account, chat_id,
-    is_chatroom, filename, date);
+  if (is_chatroom)
+    target = tpl_entity_new_from_room_id (chat_id);
+  else
+    target = tpl_entity_new (chat_id, TPL_ENTITY_CONTACT, NULL, NULL);
+
+  hit = _tpl_log_manager_search_hit_new (account, target, date);
 
   g_strfreev (strv);
   g_date_free (date);
+  g_object_unref (target);
 
   return hit;
 }
 
-/* returns a Glist of TplEntryText instances */
+/* returns a Glist of TplEvent instances */
 static GList *
-log_store_xml_get_messages_for_file (TplLogStoreXml *self,
+log_store_xml_get_events_for_file (TplLogStoreXml *self,
     TpAccount *account,
     const gchar *filename)
 {
-  GList *messages = NULL;
+  GList *events = NULL;
   xmlParserCtxtPtr ctxt;
   xmlDocPtr doc;
   xmlNodePtr log_node;
   xmlNodePtr node;
+  gboolean is_room;
+  gchar *dirname;
+  gchar *tmp;
+  gchar *target_id;
+  gchar *self_id;
+  GError *error = NULL;
 
   g_return_val_if_fail (TPL_IS_LOG_STORE_XML (self), NULL);
   g_return_val_if_fail (TP_IS_ACCOUNT (account), NULL);
@@ -784,6 +795,16 @@ log_store_xml_get_messages_for_file (TplLogStoreXml *self,
       return NULL;
     }
 
+  if (!tp_account_parse_object_path (
+        tp_proxy_get_object_path (TP_PROXY (account)),
+        NULL, NULL, &self_id, &error))
+    {
+      DEBUG ("Cannot get self identifier from account: %s",
+          error->message);
+      g_error_free (error);
+      return NULL;
+    }
+
   /* Create parser. */
   ctxt = xmlNewParserCtxt ();
 
@@ -793,6 +814,7 @@ log_store_xml_get_messages_for_file (TplLogStoreXml *self,
     {
       g_warning ("Failed to parse file:'%s'", filename);
       xmlFreeParserCtxt (ctxt);
+      g_free (self_id);
       return NULL;
     }
 
@@ -802,16 +824,31 @@ log_store_xml_get_messages_for_file (TplLogStoreXml *self,
     {
       xmlFreeDoc (doc);
       xmlFreeParserCtxt (ctxt);
+      g_free (self_id);
       return NULL;
     }
 
-  /* Now get the messages. */
+  /* Guess the target based on directory name */
+  dirname = g_path_get_dirname (filename);
+  target_id = g_path_get_basename (dirname);
+
+  /* Determine if it's a chatroom */
+  tmp = dirname;
+  dirname = g_path_get_dirname (tmp);
+  g_free (tmp);
+  tmp = g_path_get_basename (dirname);
+  is_room = (g_strcmp0 (LOG_DIR_CHATROOMS, tmp) == 0);
+  g_free (dirname);
+  g_free (tmp);
+
+  /* Now get the events. */
   for (node = log_node->children; node; node = node->next)
     {
-      TplEntryText *message;
+      TplTextEvent *event;
       TplEntity *sender;
-      gchar *time_;
-      time_t t;
+      TplEntity *receiver;
+      gchar *time_str;
+      gint64 timestamp;
       gchar *sender_id;
       gchar *sender_name;
       gchar *sender_avatar_token;
@@ -819,76 +856,57 @@ log_store_xml_get_messages_for_file (TplLogStoreXml *self,
       gchar *is_user_str;
       gboolean is_user = FALSE;
       gchar *msg_type_str;
-      gchar *log_id;
-      guint pending_id;
       TpChannelTextMessageType msg_type = TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL;
 
       if (strcmp ((const gchar *) node->name, "message") != 0)
         continue;
 
       body = (gchar *) xmlNodeGetContent (node);
-      time_ = (gchar *) xmlGetProp (node, (const xmlChar *) "time");
+      time_str = (gchar *) xmlGetProp (node, (const xmlChar *) "time");
       sender_id = (gchar *) xmlGetProp (node, (const xmlChar *) "id");
       sender_name = (gchar *) xmlGetProp (node, (const xmlChar *) "name");
       sender_avatar_token = (gchar *) xmlGetProp (node,
           (const xmlChar *) "token");
       is_user_str = (gchar *) xmlGetProp (node, (const xmlChar *) "isuser");
       msg_type_str = (gchar *) xmlGetProp (node, (const xmlChar *) "type");
-      /* in XML the attr is still cm_id to keep legacy Empathy LogStore
-       * compatibility, but actually it stores the log-id when not in
-       * legacy-mode */
-      log_id = (gchar *) xmlGetProp (node, (const xmlChar *) "cm_id");
 
       if (is_user_str != NULL)
         is_user = (!tp_strdiff (is_user_str, "true"));
 
       if (msg_type_str != NULL)
-        msg_type = _tpl_entry_text_message_type_from_str (msg_type_str);
+        msg_type = _tpl_text_event_message_type_from_str (msg_type_str);
 
-      if (log_id != NULL && self->priv->empathy_legacy)
-        /* in legacy mode, it's actually the pending message id before ACK */
-        pending_id = atoi (log_id);
+      timestamp = _tpl_time_parse (time_str);
+
+      if (is_room)
+        receiver = tpl_entity_new_from_room_id (target_id);
+      else if (is_user)
+        receiver = tpl_entity_new (target_id, TPL_ENTITY_CONTACT, NULL, NULL);
       else
-        /* we have no way in non empathy-legacy mode to know it */
-        pending_id = TPL_ENTRY_MSG_ID_UNKNOWN;
+        receiver = tpl_entity_new (self_id, TPL_ENTITY_SELF,
+            tp_account_get_nickname (account), NULL);
 
-      t = _tpl_time_parse (time_);
+      sender = tpl_entity_new (sender_id,
+          is_user ? TPL_ENTITY_SELF : TPL_ENTITY_CONTACT,
+          sender_name, sender_avatar_token);
 
-      sender = _tpl_entity_new (sender_id);
-      _tpl_entity_set_alias (sender, sender_name);
-      _tpl_entity_set_avatar_token (sender, sender_avatar_token);
+      event = g_object_new (TPL_TYPE_TEXT_EVENT,
+          /* TplEvent */
+          "account", account,
+          /* MISSING: "channel-path", channel_path, */
+          "receiver", receiver,
+          "sender", sender,
+          "timestamp", timestamp,
+          /* TplTextEvent */
+          "message-type", msg_type,
+          "message", body,
+          NULL);
 
-      if (self->priv->empathy_legacy)
-        {
-          /* in legacy Empathy LogStore there is no concept of log-id as a unique
-           * token, so I'll create, just for it to be present, an ad hoc unique
-           * token. */
-          gchar *instead_of_channel_path;
-
-          instead_of_channel_path = g_strconcat (
-              tp_proxy_get_object_path (account), sender_id, NULL);
-
-          xmlFree (log_id);
-          log_id = _tpl_create_message_token (instead_of_channel_path, t,
-              pending_id);
-
-          g_free (instead_of_channel_path);
-        }
-
-      message = _tpl_entry_text_new (log_id, account, TPL_ENTRY_DIRECTION_NONE);
-
-      _tpl_entry_text_set_pending_msg_id (TPL_ENTRY_TEXT (message),
-          pending_id);
-      _tpl_entry_set_sender (TPL_ENTRY (message), sender);
-      _tpl_entry_set_timestamp (TPL_ENTRY (message), t);
-      _tpl_entry_text_set_message (message, body);
-      _tpl_entry_text_set_message_type (message, msg_type);
-
-      messages = g_list_append (messages, message);
+      events = g_list_append (events, event);
 
       g_object_unref (sender);
-      xmlFree (log_id);
-      xmlFree (time_);
+      g_object_unref (receiver);
+      xmlFree (time_str);
       xmlFree (sender_id);
       xmlFree (sender_name);
       xmlFree (body);
@@ -897,12 +915,13 @@ log_store_xml_get_messages_for_file (TplLogStoreXml *self,
       xmlFree (sender_avatar_token);
     }
 
-  DEBUG ("Parsed %d messages", g_list_length (messages));
+  DEBUG ("Parsed %d events", g_list_length (events));
 
+  g_free (target_id);
   xmlFreeDoc (doc);
   xmlFreeParserCtxt (ctxt);
 
-  return messages;
+  return events;
 }
 
 
@@ -962,11 +981,13 @@ _log_store_xml_search_in_files (TplLogStoreXml *self,
   GList *l;
   GList *hits = NULL;
   gchar *text_casefold;
+  gchar *escaped_text;
 
   g_return_val_if_fail (TPL_IS_LOG_STORE_XML (self), NULL);
   g_return_val_if_fail (!TPL_STR_EMPTY (text), NULL);
 
-  text_casefold = g_utf8_casefold (text, -1);
+  escaped_text = g_markup_escape_text (text, -1);
+  text_casefold = g_utf8_casefold (escaped_text, -1);
 
   for (l = files; l; l = g_list_next (l))
     {
@@ -999,7 +1020,7 @@ _log_store_xml_search_in_files (TplLogStoreXml *self,
             {
               hits = g_list_prepend (hits, hit);
               DEBUG ("Found text:'%s' in file:'%s' on date: %04u-%02u-%02u",
-                  text, hit->filename, g_date_get_year (hit->date),
+                  text, filename, g_date_get_year (hit->date),
                   g_date_get_month (hit->date), g_date_get_day (hit->date));
             }
         }
@@ -1014,41 +1035,16 @@ fail:
 
   g_list_free (files);
   g_free (text_casefold);
+  g_free (escaped_text);
 
   return hits;
 }
 
 
-static GList *
-log_store_xml_search_in_identifier_chats_new (TplLogStore *store,
-    TpAccount *account,
-    const gchar *identifier,
-    const gchar *text)
-{
-  TplLogStoreXml *self = (TplLogStoreXml *) store;
-  GList *files;
-  gchar *dir, *account_dir;
-
-  g_return_val_if_fail (TPL_IS_LOG_STORE_XML (self), NULL);
-  g_return_val_if_fail (TP_IS_ACCOUNT (account), NULL);
-  g_return_val_if_fail (!TPL_STR_EMPTY (identifier), NULL);
-  g_return_val_if_fail (!TPL_STR_EMPTY (text), NULL);
-
-  account_dir = log_store_account_to_dirname (account);
-  dir = g_build_path (G_DIR_SEPARATOR_S, log_store_xml_get_basedir (self),
-      account_dir, identifier, NULL);
-
-  files = log_store_xml_get_all_files (self, dir);
-  DEBUG ("Found %d log files in total", g_list_length (files));
-
-  return _log_store_xml_search_in_files (self, text, files);
-}
-
-
-
 static GList *
 log_store_xml_search_new (TplLogStore *store,
-    const gchar *text)
+    const gchar *text,
+    gint type_mask)
 {
   TplLogStoreXml *self = (TplLogStoreXml *) store;
   GList *files;
@@ -1056,6 +1052,9 @@ log_store_xml_search_new (TplLogStore *store,
   g_return_val_if_fail (TPL_IS_LOG_STORE_XML (self), NULL);
   g_return_val_if_fail (!TPL_STR_EMPTY (text), NULL);
 
+  if (!(type_mask & TPL_EVENT_MASK_TEXT))
+    return NULL;
+
   files = log_store_xml_get_all_files (self, NULL);
   DEBUG ("Found %d log files in total", g_list_length (files));
 
@@ -1064,13 +1063,13 @@ log_store_xml_search_new (TplLogStore *store,
 
 /* Returns: (GList *) of (TplLogSearchHit *) */
 static GList *
-log_store_xml_get_chats_for_dir (TplLogStoreXml *self,
+log_store_xml_get_entities_for_dir (TplLogStoreXml *self,
     const gchar *dir,
     gboolean is_chatroom,
     TpAccount *account)
 {
   GDir *gdir;
-  GList *hits = NULL;
+  GList *entities = NULL;
   const gchar *name;
   GError *error = NULL;
 
@@ -1087,72 +1086,76 @@ log_store_xml_get_chats_for_dir (TplLogStoreXml *self,
 
   while ((name = g_dir_read_name (gdir)) != NULL)
     {
-      TplLogSearchHit *hit;
+      TplEntity *entity;
 
       if (!is_chatroom && strcmp (name, LOG_DIR_CHATROOMS) == 0)
         {
           gchar *filename = g_build_filename (dir, name, NULL);
-          hits = g_list_concat (hits,
-              log_store_xml_get_chats_for_dir (self, filename, TRUE, account));
+          entities = g_list_concat (entities,
+              log_store_xml_get_entities_for_dir (self, filename, TRUE, account));
           g_free (filename);
           continue;
         }
 
-      hit = _tpl_log_manager_search_hit_new (account, name, is_chatroom,
-          NULL, NULL);
+      if (is_chatroom)
+        entity = tpl_entity_new_from_room_id (name);
+      else
+        entity = tpl_entity_new (name, TPL_ENTITY_CONTACT, NULL, NULL);
 
-      hits = g_list_prepend (hits, hit);
+      entities = g_list_prepend (entities, entity);
     }
 
   g_dir_close (gdir);
 
-  return hits;
+  return entities;
 }
 
 
-/* returns a Glist of TplEntryText instances */
+/* returns a Glist of TplEvent instances */
 static GList *
-log_store_xml_get_messages_for_date (TplLogStore *store,
+log_store_xml_get_events_for_date (TplLogStore *store,
     TpAccount *account,
-    const gchar *chat_id,
-    gboolean chatroom,
+    TplEntity *target,
+    gint type_mask,
     const GDate *date)
 {
   TplLogStoreXml *self = (TplLogStoreXml *) store;
   gchar *filename;
-  GList *messages;
+  GList *events;
 
   g_return_val_if_fail (TPL_IS_LOG_STORE_XML (self), NULL);
   g_return_val_if_fail (TP_IS_ACCOUNT (account), NULL);
-  g_return_val_if_fail (!TPL_STR_EMPTY (chat_id), NULL);
+  g_return_val_if_fail (TPL_IS_ENTITY (target), NULL);
   g_return_val_if_fail (date != NULL, NULL);
 
-  filename = log_store_xml_get_filename_for_date (self, account, chat_id,
-      chatroom, date);
-  messages = log_store_xml_get_messages_for_file (self, account,
-      filename);
+  if (!(type_mask & TPL_EVENT_MASK_TEXT))
+    return NULL;
+
+  filename = log_store_xml_get_filename_for_date (self, account, target,
+      date);
+  events = log_store_xml_get_events_for_file (self, account, filename);
   g_free (filename);
 
-  return messages;
+  return events;
 }
 
 
 static GList *
-log_store_xml_get_chats (TplLogStore *store,
+log_store_xml_get_entities (TplLogStore *store,
     TpAccount *account)
 {
   TplLogStoreXml *self = (TplLogStoreXml *) store;
   gchar *dir;
-  GList *hits;
+  GList *entities;
 
   g_return_val_if_fail (TPL_IS_LOG_STORE_XML (self), NULL);
   g_return_val_if_fail (TP_IS_ACCOUNT (account), NULL);
 
-  dir = log_store_xml_get_dir (self, account, NULL, FALSE);
-  hits = log_store_xml_get_chats_for_dir (self, dir, FALSE, account);
+  dir = log_store_xml_get_dir (self, account, NULL);
+  entities = log_store_xml_get_entities_for_dir (self, dir, FALSE, account);
   g_free (dir);
 
-  return hits;
+  return entities;
 }
 
 
@@ -1178,9 +1181,22 @@ log_store_xml_get_basedir (TplLogStoreXml *self)
   if (self->priv->basedir == NULL)
     {
       gchar *dir;
+      const char *user_data_dir;
+      const char *name;
 
-      dir = g_build_path (G_DIR_SEPARATOR_S, g_get_user_data_dir (),
-          log_store_xml_get_name ((TplLogStore *) self), "logs", NULL);
+      if (self->priv->test_mode && g_getenv ("TPL_TEST_LOG_DIR") != NULL)
+        {
+          user_data_dir = g_getenv ("TPL_TEST_LOG_DIR");
+          name = self->priv->empathy_legacy ? "Empathy" : "TpLogger";
+        }
+      else
+        {
+          user_data_dir = g_get_user_data_dir ();
+          name =  log_store_xml_get_name ((TplLogStore *) self);
+        }
+
+      dir = g_build_path (G_DIR_SEPARATOR_S, user_data_dir, name, "logs",
+          NULL);
       log_store_xml_set_basedir (self, dir);
       g_free (dir);
     }
@@ -1238,56 +1254,117 @@ log_store_xml_set_writable (TplLogStoreXml *self,
 
 
 static GList *
-log_store_xml_get_filtered_messages (TplLogStore *store,
+log_store_xml_get_filtered_events (TplLogStore *store,
     TpAccount *account,
-    const gchar *chat_id,
-    gboolean chatroom,
-    guint num_messages,
-    TplLogMessageFilter filter,
+    TplEntity *target,
+    gint type_mask,
+    guint num_events,
+    TplLogEventFilter filter,
     gpointer user_data)
 {
   TplLogStoreXml *self = (TplLogStoreXml *) store;
-  GList *dates, *l, *messages = NULL;
+  GList *dates, *l, *events = NULL;
   guint i = 0;
 
   g_return_val_if_fail (TPL_IS_LOG_STORE_XML (self), NULL);
   g_return_val_if_fail (TP_IS_ACCOUNT (account), NULL);
-  g_return_val_if_fail (!TPL_STR_EMPTY (chat_id), NULL);
+  g_return_val_if_fail (TPL_IS_ENTITY (target), NULL);
 
-  dates = log_store_xml_get_dates (store, account, chat_id, chatroom);
+  dates = log_store_xml_get_dates (store, account, target, type_mask);
 
-  for (l = g_list_last (dates); l != NULL && i < num_messages;
+  for (l = g_list_last (dates); l != NULL && i < num_events;
        l = g_list_previous (l))
     {
-      GList *new_messages, *n, *next;
+      GList *new_events, *n;
 
-      /* FIXME: We should really restrict the message parsing to get only
-       * the newest num_messages. */
-      new_messages = log_store_xml_get_messages_for_date (store, account,
-          chat_id, chatroom, l->data);
+      /* FIXME: We should really restrict the event parsing to get only
+       * the newest num_events. */
+      new_events = log_store_xml_get_events_for_date (store, account,
+          target, type_mask, l->data);
 
-      n = new_messages;
-      while (n != NULL)
+      n = g_list_last (new_events);
+      while (n != NULL && i < num_events)
         {
-          next = g_list_next (n);
-          if (filter != NULL && !filter (n->data, user_data))
+          if (filter == NULL || filter (n->data, user_data))
             {
-              g_object_unref (n->data);
-              new_messages = g_list_delete_link (new_messages, n);
+              events = g_list_prepend (events, g_object_ref (n->data));
+              i++;
             }
-          else
-            i++;
-          n = next;
+          n = g_list_previous (n);
         }
-      messages = g_list_concat (messages, new_messages);
+      g_list_foreach (new_events, (GFunc) g_object_unref, NULL);
+      g_list_free (new_events);
     }
 
-  g_list_foreach (dates, (GFunc) g_free, NULL);
+  g_list_foreach (dates, (GFunc) g_date_free, NULL);
   g_list_free (dates);
 
-  return messages;
+  return events;
 }
 
+
+static void
+log_store_xml_clear (TplLogStore *store)
+{
+  TplLogStoreXml *self = TPL_LOG_STORE_XML (store);
+  const gchar *basedir;
+
+  /* We need to use the getter otherwise the basedir might not be set yet */
+  basedir = log_store_xml_get_basedir (self);
+
+  DEBUG ("Clear all logs from XML store in: %s", basedir);
+
+  _tpl_rmdir_recursively (basedir);
+}
+
+
+static void
+log_store_xml_clear_account (TplLogStore *store,
+    TpAccount *account)
+{
+  TplLogStoreXml *self = TPL_LOG_STORE_XML (store);
+  gchar *account_dir;
+
+  account_dir = log_store_xml_get_dir (self, account, NULL);
+
+  if (account_dir)
+    {
+      DEBUG ("Clear account logs from XML store in: %s",
+          account_dir);
+      _tpl_rmdir_recursively (account_dir);
+      g_free (account_dir);
+    }
+  else
+    DEBUG ("Nothing to clear in account: %s",
+        tp_proxy_get_object_path (TP_PROXY (account)));
+}
+
+
+static void
+log_store_xml_clear_entity (TplLogStore *store,
+    TpAccount *account,
+    TplEntity *entity)
+{
+  TplLogStoreXml *self = TPL_LOG_STORE_XML (store);
+  gchar *entity_dir;
+
+  entity_dir = log_store_xml_get_dir (self, account, entity);
+
+  if (entity_dir)
+    {
+      DEBUG ("Clear entity logs from XML store in: %s",
+          entity_dir);
+
+      _tpl_rmdir_recursively (entity_dir);
+      g_free (entity_dir);
+    }
+  else
+    DEBUG ("Nothing to clear for account/entity: %s/%s",
+        tp_proxy_get_object_path (TP_PROXY (account)),
+        tpl_entity_get_identifier (entity));
+}
+
+
 static void
 log_store_iface_init (gpointer g_iface,
     gpointer iface_data)
@@ -1296,12 +1373,13 @@ log_store_iface_init (gpointer g_iface,
 
   iface->get_name = log_store_xml_get_name;
   iface->exists = log_store_xml_exists;
-  iface->add_message = log_store_xml_add_message;
+  iface->add_event = log_store_xml_add_event;
   iface->get_dates = log_store_xml_get_dates;
-  iface->get_messages_for_date = log_store_xml_get_messages_for_date;
-  iface->get_chats = log_store_xml_get_chats;
-  iface->search_in_identifier_chats_new =
-    log_store_xml_search_in_identifier_chats_new;
+  iface->get_events_for_date = log_store_xml_get_events_for_date;
+  iface->get_entities = log_store_xml_get_entities;
   iface->search_new = log_store_xml_search_new;
-  iface->get_filtered_messages = log_store_xml_get_filtered_messages;
+  iface->get_filtered_events = log_store_xml_get_filtered_events;
+  iface->clear = log_store_xml_clear;
+  iface->clear_account = log_store_xml_clear_account;
+  iface->clear_entity = log_store_xml_clear_entity;
 }
